<?php if ( ! defined('BASEPATH')) exit('No direct script access allowed');

class Entity_model extends CI_Model {
	
	function __construct()
	{
        parent::__construct();
        $this->load->library(OTBASE_PATH.'OT_App');
        $this->load->library(OTBASE_PATH.'OT_User');
        $this->load->library('form_validation');

		$this->load->library('entity_accounts_table');
		$this->load->library('Auth');
		$this->cur_time = $this->auth->get_timezone_date();
		$this->error = '';
	}

	private $salt	= "viKnedJev7";

	public function save($data)
	{
		$values =  $this->entity_accounts_table->save($data);
		if ($values) {
			//  send mail
			$sub	= sprintf(lang("CLOUD_ENTITY_REGISTERED"), $values['user_name']);
			$url = $this->config->item('protocal').$values['domain_url'];
			$url = '<a href="'.$url.'" target="_blank">'.$url.'</a>';
			$msg	= sprintf(lang("CLOUD_ENTITY_REGISTER_TEMPLATE"), $values['user_name'],$values['user_name'], $url, $values['email']);
				
			$this->load->library('email');
			$email_config = array(
					'mailtype'	=> 'html',
					'charset'	=> 'utf-8'
			);
			$this->email->initialize($email_config);
			$email	= $this->email;
			$email->clear();
			$email->to($values['email']);
	        $email->from($this->config->item('smtp_user'), $this->config->item('domain_name'));
			$email->subject($sub);
			$email->message(nl2br($msg));
			if (!$email->send())
			{
				$this->error	= $email->print_debugger();
				// 				return false;
			}
		}
		return $values;	
	}

	public function register($data)
	{		
		if ($result = $this->entity_accounts_table->register($data))
		{
			$sub	= sprintf(lang("ON_PRE_ENTITY_REGISTERED"), $result['user_name']);
			$msg	= sprintf(lang("ON_PRE_ENTITY_REGISTER_TEMPLATE"), $result['user_name'], $result['user_name'],'<a href="http://outputtime.com/releases/index.php?installer=latest">http://outputtime.com/releases/index.php?installer=latest</a>',$result['license_key']);

			$this->load->library('email');
	        $this->email->clear();
	        $this->email->to($result['email']);
	        $this->email->from($this->config->item('smtp_user'), $this->config->item('domain_name'));
	        $this->email->subject($sub);
	        $this->email->message(nl2br($msg));	
     
			if (!$this->email->send())
			{
				$this->error	= $this->email->print_debugger();
// 				return false;
			}
			return true;
		}

		return false;
	}


	public function checkdomain($domain_url="", $_subdom="")
	{
		if (!empty($domain_url))
		{
			$domain_url.='.'.$this->config->item('domain');
			$this->db->select('entity_id');
			$this->db->where("find_in_set('$domain_url',domain_url) !=",0);
			$this->db->from('entity_accounts');
			$query	= $this->db->get();
			if ($query->num_rows()==0)
			{
				$subdom	= hash("sha256", $this->salt . $domain_url . $this->salt);
				return (empty($_subdom) ? ($subdom) : ($_subdom == $subdom));
			}
		}
		return false;
	}

	public function check_license($license_key='')
	{
		$this->db->select('entity_id');
		$this->db->where("license_key", $license_key);
		$this->db->from('entity_accounts');
		$query	= $this->db->get();

		if ($query->num_rows() == 1)
		{
			return true;
		}
		return false;
	}

	public function save_setting($name, $data)
	{
		$method = 'save_' . $name;
		$data	= $this->{$method}($data);
		if(!$data)
			return false;

		return $data;
	}

	function save_company($data)
	{
		$this->load->library('entity_table');
		if(!$this->entity_table->save($data))
		{
			$this->error = lang('SAVE_ERROR') . $this->entity_table->error;
			return false;	
		}
		return true;
	}

// 	public function get_license_details()
// 	{
// 		$this->db->select('license_key');
// 		$entity_id	= $this->ot_app->get('entity_id');
// 		$this->db->where("entity_id", $entity_id);
// 		$this->db->from('entity_accounts');
// 		$query	= $this->db->get();

// 		$result	= $query->row_array();

// 		if ($result && $result['license_key'])
// 		{
// 			return $this->auth->parse_license($result['license_key']);
// 		}

// 		return false;
// 	}

	function save_datetime($data)
	{
		return $this->save_general_setting($data);
	}

	function save_timesheet($data)
	{
		return $this->save_general_setting($data);
	}

	function save_invoice($data)
	{
		return $this->save_general_setting($data);
	}

	function save_calendar($data)
	{
		$this->load->library('entity_setting_table');
		$this->load->library('entity_daysoff_table');

		$entity_id	= $this->ot_app->get('entity_id');

		$en_data	= $data;
		unset($en_data['daysoff']);

		$this->entity_setting_table->tbl_key = 'entity_id';

		$return	= array("save_ids"	=> array());
		if($this->entity_setting_table->save($en_data))
		{
			if(!empty($data['daysoff']))
			{
				foreach($data['daysoff'] as $val)
				{
					$dayoff_id	= $this->entity_daysoff_table->save($val);
					if(!$dayoff_id)
					{
						$this->error = lang('SAVE_ERROR') . $this->entity_daysoff_table->error;
						return true;	
					}
					$return["save_ids"][]	= $dayoff_id;
				}
			}

			if(!empty($data['deleted_daysoff']))
			{
				if(!$this->delete_data($data['deleted_daysoff'], array('tbl_key'=>'entity_dayoff_id','tbl'=>'entity_daysoff')))
				{
					$this->error = lang('SAVE_ERROR') .  $this->error;
					return true;	
				}
			}
			return $return;
		}
	}

	function save_task_types($data)
	{
		$this->load->library('task_type_table');
		$entity_id	= $this->ot_app->get('entity_id');
		$task_data	= array();
		$return	= array(
			"save_ids"	=> array() 	
		);
		
		if(!empty($data['deleted_task_types']))
		{
			if($this->get_task_type_count($data['deleted_task_types']) > 0)
			{
				$this->error = lang('ERR_PROJECT_TASK_TYPE_LINK') . $this->error;
				return false;
			}
			if(!$this->delete_data($data['deleted_task_types'], array('tbl_key'=>'task_type_id','tbl'=>'task_type')))
			{
				$this->error = lang('SAVE_ERROR') . $this->error;
				return false;
			}
		}

		if(!empty($data['items']))
		{
			foreach($data['items'] as $task_type)
			{
				$task_data	= $task_type;
				$task_data['entity_id']	= $entity_id;
				if($task_data['task_type_id'] <= 0)
				{
					$max_sort_order = $this->get_last_sort_order('task_type', $entity_id);
					$task_data['sort_order'] = $max_sort_order + 1;
				}
				else
					unset($task_data['sort_order']);

				$this->db->select('task_type_id, name');
				$this->db->where('name', $task_data['name']);
				$this->db->where('entity_id', $entity_id);
				$query = $this->db->get('task_type');
				$result = $query->row_array();
				
				if($result && ($result['task_type_id'] != $task_data['task_type_id'] && $result['name'] == $task_data['name'])) 
				{
					$this->error	= lang('DUPLICATE_ENTRY'); 
					return false;
				}

				if(!$result || $result['task_type_id'] == $task_data['task_type_id'])
				{
					$task_id = $this->task_type_table->save($task_data);

					if(!$task_id)
					{
						$this->error = lang('SAVE_ERROR');// . (!empty($this->task_type_table->error))?$this->task_type_table->error:"";
						return false;
					}
					if($task_data['task_type_id'] <= 0)
					{
						$return["save_ids"][]	= $task_id;
					}
				}
			}
		}

		return $return;
	}

	function get_task_type_count($ids)
	{
		$sql = 'SELECT EXISTS (SELECT 1 FROM ot_project p INNER JOIN ot_project_task_types ptt ON p.project_id = ptt.project_id WHERE p.entity_id =' . $this->ot_app->get('entity_id') . ' AND ptt.task_type_id in(' . implode($ids, ",") . ')) AS count FROM ot_task_type LIMIT 1';
		$query = $this->db->query($sql);
		$result = $query->row_array();
		return (isset($result)?$result['count']:0);
		
	}
	function get_task_status_count($ids)
	{
		$sql = 'SELECT EXISTS(SELECT 1 from ot_project_task pt where pt.entity_id=' . $this->ot_app->get('entity_id') . '
				and task_status_id in (' . implode($ids, ",") . ')) as count FROM ot_task_status limit 1';
		$query = $this->db->query($sql);
		$result = $query->row_array();
		return (isset($result)?$result['count']:0);
	}
	function get_default_task_status($ids)
	{
		$this->db->select('count(*) as total_count');
		$this->db->where('system_defined',1);
		$this->db->where_in('task_status_id', $ids);
		$query = $this->db->get('task_status');
		$result = $query->row_array();
		return (isset($result['total_count'])?$result['total_count']:0);
	}
	function get_expense_type_count($ids)
	{
		$sql = 'SELECT EXISTS( SELECT 1 from ot_project p INNER JOIN ot_project_expense pe on p.project_id=pe.project_id where
				p.entity_id=' . $this->ot_app->get('entity_id') . '  and expense_type_id in(' . implode($ids, ",") . ')) as count FROM ot_expense_type limit 1';

		$query = $this->db->query($sql);
		$result = $query->row_array();
		return (isset($result)?$result['count']:0);
	}
	function get_last_sort_order($tbl, $entity_id)
	{
		$this->db->select('max(sort_order) as sort_order');
		$this->db->where('entity_id',$entity_id);
		$query = $this->db->get($tbl);
		$result = $query->row_array();
		return $result['sort_order'];
		
	}
	function save_expense_types($data)
	{
		$this->load->library('expense_type_table');
		$entity_id	= $this->ot_app->get('entity_id');
		$return	= array(
			"save_ids"	=> array() 	
		);
		
		if(!empty($data['deleted_expenses_types']))
		{
			if($this->get_expense_type_count($data['deleted_expenses_types'])>0)
			{
				$this->error =  lang('ERR_PROJECT_EXPENSE_TYPE_LINK');
				return false;
			}

			if(!$this->delete_data($data['deleted_expenses_types'], array('tbl_key'=>'expense_type_id','tbl'=>'expense_type')))
			{
				$this->error = lang('SAVE_ERROR') . $this->error;
				return false;
			}
		}

		if(!empty($data['items']))
		{
			$exp_data	= array();
			foreach($data['items'] as $expense_type)
			{
				$exp_data	= $expense_type;

				$exp_data['entity_id']	= $entity_id;

				$this->db->select('expense_type_id, name');
				$this->db->where('name', $exp_data['name']);
				$this->db->where('entity_id', $entity_id);
				$query = $this->db->get('expense_type');
				$result = $query->row_array();
				
				if($result && ($result['expense_type_id'] != $exp_data['expense_type_id'] && $result['name'] == $exp_data['name'])) 
				{
					$this->error	= lang('DUPLICATE_ENTRY'); 
					return false;
				}

				if(!$result || $result['expense_type_id'] == $exp_data['expense_type_id'])
				{
					$exp_id = $this->expense_type_table->save($exp_data);
					if(!$exp_id)
					{
						$this->error = lang('SAVE_ERROR');
						return false;	
					}
					if($exp_data['expense_type_id'] <= 0)
					{
						$return["save_ids"][]	= $exp_id;
					}
				}
			}
		}
		return $return;
	}

	function save_departments($data)
	{
		$this->load->library('user_dept_table');
		$entity_id	= $this->ot_app->get('entity_id');
		$return	= array(
			"save_ids"	=> array() 	
		);
		if(!empty($data['deleted_departments']))
		{
			if(!$this->delete_data($data['deleted_departments'], array('tbl_key'=>'user_dept_id','tbl'=>'user_department')))
			{
				$this->error = lang('SAVE_ERROR') .  $this->error;
				return false;
			}
		}
		if(!empty($data['items']))
		{
			$dept_data	= array();
			foreach($data['items'] as $dept)
			{
				$dept_data	= $dept;
				$dept_data['entity_id']	= $entity_id;

				$this->db->select('user_dept_id, department');
				$this->db->where('department',$dept_data['department']);
				$this->db->where('entity_id', $dept_data['entity_id']);
				$query = $this->db->get('user_department');
				$result = $query->row_array();
				
				if($result && ($result['user_dept_id'] != $dept_data['user_dept_id'] && $result['department'] == $dept_data['department'])) 
				{
					$this->error	= lang('DUPLICATE_ENTRY'); 
					return false;
				}
		
				if(!$result || $result['user_dept_id'] == $dept_data['user_dept_id'])
				{					
					$dept_id	= $this->user_dept_table->save($dept_data);
					if(!$dept_id)
					{
						$this->error = lang('SAVE_ERROR') . $this->user_dept_table->error;
						return false;	
					}
					if($dept_data['user_dept_id'] <= 0)
					{
						$return["save_ids"][]	= $dept_id;
					}
				}
			}
		}	

		

		return $return;
	}
	function save_task_status($data)
	{
		$this->load->library('task_status_table');
		$entity_id	= $this->ot_app->get('entity_id');

		$return	= array(
			"save_ids"	=> array() 	
		);

		if(!empty($data['deleted_task_status']))
		{
			if($this->get_task_status_count($data['deleted_task_status']) > 0)
			{
				$this->error =  lang('ERR_PROJECT_TASK_STATUS_LINK');
				return false;	
			}

			if($this->get_default_task_status($data['deleted_task_status']) > 0)
			{
				$this->error =  lang('ERR_DEFAULT_STATUS_NOT_DELETE');
				return false;	
			}

			if(!$this->delete_data($data['deleted_task_status'], array('tbl_key'=>'task_status_id','tbl'=>'task_status')))
			{
				$this->error = lang('SAVE_ERROR') .  $this->error;
				return false;
			}
		}
		
		if(!empty($data['items']))
		{
			$status_data['entity_id']	= $entity_id;

//			$this->db->update('task_status', array('is_default'=>'0'), array('entity_id'=>$entity_id)); 

			$default_task = 0;
			foreach($data['items'] as $task_status)
			{
				$status_data	= $task_status;
				$status_data['entity_id']	= $entity_id;

				if($task_status['task_status_id'] <= 0)
				{
					$max_sort_order = $this->get_last_sort_order('task_status',$this->ot_app->get('entity_id'));
					$status_data['sort_order'] = $max_sort_order+1;
				}
				else
					unset($status_data['sort_order']);

				$this->db->select('task_status_id, name');
				$this->db->where('name',$status_data['name']);
				$this->db->where('entity_id',$status_data['entity_id']);
				$query = $this->db->get('task_status');
				$result = $query->row_array();
				
				if($result && ($result['task_status_id'] != $status_data['task_status_id'] && $result['name'] == $status_data['name'])) 
				{
					$this->error	= lang('DUPLICATE_ENTRY'); 
					return false;
				}
		
				if(!$result || $result['task_status_id'] == $status_data['task_status_id'])
				{
					
					$status_id	= $this->task_status_table->save($status_data);

					if(!$status_id)
					{
						$this->error	= lang('SAVE_ERROR') . $this->task_status_table->error;
						return false;	
					}
					// get default task status id
					if (!$default_task && !empty($status_data['is_default']))
					{
						$default_task = $status_id;
					}
					
					if($status_data['task_status_id'] <= 0)
					{
						$return["save_ids"][]	= $status_id;
					}
				}
			}

			if ($default_task)
				$this->db->update('task_status', array('is_default'=>'0'), array('entity_id'=>$entity_id, 'task_status_id !='=> $default_task)); 
		}

		

		return $return;
		
	}
	function save_general_setting($data)
	{
		$this->load->library('entity_setting_table');

		$data['entity_id'] 			= $this->ot_app->get('entity_id');
		$data['last_updated_date'] 	= $this->cur_time;

		$this->entity_setting_table->tbl_key 	= 'entity_id';

		return $this->entity_setting_table->save($data);
	}
	
	function get_general_settings($fields = '')
	{
		if(empty($fields))
		{
			$this->db->select('ea.company_name, concat(eu.first_name," ",eu.last_name) as user_name, eu.user_image, e.entity_logo, es.time_zone_code, es.date_format, es.time_format, es.period_format, es.time_stamp_req, es.time_sheet_approval, es.modify_billable, es.time_round_to, es.fiscal_year_month, es.currency_code, es.number_format, cu.currency_symbol, es.account_owner_id, es.inv_template,es.hours_per_day', false);
		}
		else
		{
			$this->db->select($fields);
		}

		$entity	= $this->ot_app->get('entity_id');

		$this->db->from('entity_setting AS es');
		$this->db->join('currency AS cu','es.currency_code=cu.currency_code','inner');
		$this->db->join('entity_users AS eu','eu.entity_id=es.entity_id and eu.user_id=es.account_owner_id','inner');
		$this->db->join('entity_accounts AS ea','eu.entity_id=ea.entity_id','inner');
		$this->db->join('entity AS e','e.entity_id=es.entity_id','inner');

		$this->db->where('es.entity_id', $entity);

//		if ($owner)
//		{
//			$this->db->where('es.account_owner_id', $this->ot_user->get('user_id'));
//		}
		$query = $this->db->get();	

		$result = $query->row_array();

		if(!empty($result['user_image']))
		{
			$result['user_image']	= $result['user_image'];
		}

		$result['entity_logo_disp'] = 'files/photo/nocompany.jpg';
		if(!empty($result['entity_logo']))
		{
			$file	= pathinfo($result['entity_logo']);
			$_file_name	= 'uploads/' . $entity . '/' . $file['filename'] . '_160' . (empty($file['extension']) ? '' : ('.'.$file['extension']));
			if (file_exists(FCPATH . $_file_name))
			{
				$result['entity_logo_disp']	= $_file_name;
			}
		}
		return $result;
	}
	
	function get_administrator_users()
	{
		$result 	= array();
		$entity_id	= $this->ot_app->get('entity_id');

		$this->db->select('user_id, first_name, last_name, user_image');
		$this->db->where(array('entity_id' => $entity_id, 'is_active' => 1, 'user_role'=>'A'));
		$query = $this->db->get('entity_users');
		$result = $query->result_array();
		
		return $result;
	}
	
	function get_timezone_details($timezone)
	{
		$result = array();
		
		$this->db->select('description');
		$this->db->where('time_zone_code',$timezone);
		$query = $this->db->get('time_zone');
		$time_zone = $query->row_array();
		
		if($time_zone['description']!='') {
			preg_match('/\((.*?)\)/', $time_zone['description'], $matches);
			$time_zones = explode($matches[0], $time_zone['description']);
			$result['time_zone_lbl'] = $matches[0];
			$result['time_zone_city'] = $time_zones[1];
		}
		
		return $result;
	}
	function get_company()
	{
		$result = $this->get_general_settings('e.name as company_name, e.domain_name as user_name, e.entity_logo as image, eu.user_image, es.account_owner_id ');
		return $result;
	}
	function get_datetime()
	{
		$result = $this->get_general_settings('es.time_zone_code as time_zone, es.date_format, es.time_format');
		return $result;
	}
	function get_timesheet()
	{
		$result = $this->get_general_settings('es.time_stamp_req as is_required, es.time_sheet_approval, es.modify_billable');
		return $result;
	}
	function get_invoice()
	{
		$result = $this->get_general_settings('es.fiscal_year_month as fiscal_year_start, es.currency_code as currency, es.number_format as number_format,es.time_round_to as round_time,cu.currency_symbol');
		return $result;
	}

	function get_calendar()
	{
		$entity_id	= $this->ot_app->get('entity_id');

		$this->db->select('start_week_on, non_working_days, skip_daysoff');
		$this->db->where('entity_id', $entity_id);

		$query	= $this->db->get('entity_setting');
		$result	= $query->row_array();

		// Getting days off details
		$this->db->select("entity_dayoff_id, description, repeat, dayoff_date");
		$this->db->select("DATE_FORMAT(dayoff_date, '%m-%d') AS month_date", false);
		$this->db->where('entity_id', $entity_id);
		$this->db->where('(year(dayoff_date)="' . date('Y') . '" OR `repeat`="1")', '', false);
		$this->db->order_by('month_date');
		$days_query 	= $this->db->get('entity_daysoff');
		$daysoff_result	= $days_query->result_array();

		$result['daysoff']	= array();
		if (count($daysoff_result))
		{
			$result['daysoff']	= $this->response->prepare($daysoff_result);
		}

		return $result;
	}

	function get_task_types($select = 'task_type_id, name, hourly_rate, is_common, sort_order, is_billable', $options = array())
	{
		$this->db->select($select);
		$where = array();
		if (!empty($options['filters']))
		{
			foreach($options['filters'] as $key=>$val) {
					if (is_array($val))
						$this->db->where_in($key,$val);
					else
						$where[$key] = $val;
			}			
		}
		$where = array_merge($where, array('entity_id' => $this->ot_app->get('entity_id')));
		$this->db->where($where);
		$this->db->order_by('sort_order');
		$query = $this->db->get('task_type');
		
		return $query->result_array();
	}
	function get_expense_types ()
	{
		$this->db->select('expense_type_id, name, unit_price, units');
		$this->db->where('entity_id',$this->ot_app->get('entity_id'));
		$this->db->order_by('name');
		$query = $this->db->get('expense_type');
		$result = $query->result_array();
		return $result;
	}

	function get_task_status($select = '')
	{
		$select = $select?$select:'task_status_id, name, text_color, bg_color, is_default, system_defined, is_closed';
		$this->db->select($select);
		$this->db->where('entity_id',$this->ot_app->get('entity_id'));
		$this->db->order_by('sort_order');
		$query	= $this->db->get('task_status');
		$result	= $query->result_array();

		return $result;
	}
	function get_departments()
	{
		$this->db->select('user_dept_id, department');
		$this->db->where('entity_id',$this->ot_app->get('entity_id'));
		$this->db->order_by('department');
		$query = $this->db->get('user_department');
		$result = $query->result_array();
		return $result;	
	}
	
	function get_addons()
	{
		$this->db->select('ap.app_name, ap.developed_by, ap.app_description, ap.is_active, ape.is_enabled');
		$this->db->from('apps AS ap');
		$this->db->join('apps_entity AS ape', 'ap.app_id=ape.app_id AND ape.entity_id='.$this->ot_app->get('entity_id'),'left');
		$this->db->where(array('ap.is_active' => 1));
		$query = $this->db->get();
		$result = $query->result_array();
		return $result;	
	}
			
	public function get_settings($name)
	{
		$user_id	= $this->ot_user->get('user_id');
		$entity_id	= $this->ot_app->get('entity_id');
		
		$method = 'get_' . $name;
		$result = $this->{$method}();
		if($result)
			return $result;

		return false;
	
	}
	public function get_timezone()
	{
		$this->db->select('time_zone_code, description');
		$this->db->order_by('description');
		$query = $this->db->get('time_zone');
		$result = $query->result_array();
		return $result;
	}
	public function get_currency()
	{
		$this->db->select('currency_code, description, currency_symbol');
		$this->db->order_by('description');
		$query = $this->db->get('currency');
		$result = $query->result_array();
		return $result;
	}
	public function get_data($tbl, $tbl_key)
	{
		$this->db->select($tbl_key);
		$this->db->where('entity_id',$this->ot_app->get('entity_id'));
		$query = $this->db->get($tbl);
		$result = $query->result_array();
		$result_ids = array();
		if(!empty($result)) {
			foreach($result as $key=>$value) {
				$result_ids[] = $value[$tbl_key];
			}
		}
		return $result_ids;
	}
	public function delete_data($ids = array(), $data)
	{
		if(!empty($ids)) {
			$this->db->where_in($data['tbl_key'], $ids);
			return $this->db->delete($data['tbl']);
		}
		return true;
	}
	function task_type_name_check($value, $task_type_id)
	{
		$fields['name'] = $value;
		$fields['entity_id'] = $this->ot_app->get('entity_id');
		if($task_type_id>0) $fields['task_type_id !=']	= $task_type_id;
		$query = $this->db->limit(1)->get_where('task_type', $fields);
		
		return $query->num_rows();
	}
	
	function sort_items ($data, $t_name, $f_name, $or_name)
	{
		if(!empty($data))
		{
			$sort_order = 1;
			foreach($data['items'] as $value)
			{
				$this->db->where(array($f_name => $value[$f_name]));
				if(!$this->db->update($t_name, array($or_name => $sort_order)))
				{
					return false;
				}
				$sort_order++;
			}
		}
		return true;
	}
	public function get_entity_admin($select='')
	{
		$entity_id = $this->ot_app->get('entity_id');
		$select = ($select!='') ? $select : '*'; 
		$this->db->select($select);
		$this->db->from('entity_users AS eu');
		$this->db->join('entity_setting AS es','eu.entity_id=es.entity_id and es.account_owner_id=eu.user_id','inner');
		$this->db->where(array('eu.entity_id' => $entity_id));
		$query = $this->db->get();
		
		$result = $query->row_array();
		return $result;
	}
	public function get_invoice_template() {
		$this->load->helper('file');
		
		$templates = get_filenames(APPPATH.'views'.DIRECTORY_SEPARATOR.'invoice_templates');
		if ($templates) {
			foreach ($templates as $key => $val) {
				$templates[$key] = basename($val,'.php');
			}
		}
		
		return $templates;
	}
	public function get_active_inv_template() {
		$this->db->select('inv_template');
		$this->db->from('entity_setting');
		$this->db->where(array('entity_id' => $this->ot_app->get('entity_id')));
		$query = $this->db->get();
		
		$result = $query->row_array();
		return $result;
	}

	public function get_entity_counts() {
		return $this->db->count_all('entity');
	}

	public function get_entity_details($entity_id='',$select='entity_id, company_name, domain_url, user_name, email, exp_date, max_users, plan')
	{
		$entity_id	= empty($entity_id) ? $this->ot_app->get('entity_id') : $entity_id;

		$user_count	= $this->db->select('count(user_id) as user_count')->from('entity_users')->where('entity_id', $entity_id)->get()->row_array();
		$return		= $this->db->select($select)->from('entity_accounts')->where('entity_id', $entity_id)->get()->row_array();
		$return['user_count']	= $user_count['user_count'];

		return $return;
	}

	public function update_entity_license($data=array())
	{
		$this->db->where('entity_id', $data['entity_id']);
		$store_data = array(
			'max_users'	=> $data['max_users'],
			'exp_date'	=> $data['exp_date'],
			'plan'	=> $data['plan']
		);
		if (!empty($data['license_key']))
		{
			$store_data['license_key']	= $data['license_key'];
		}
		$this->db->update('entity_accounts', $store_data);
	}

	function forgot_domain ($to_email) 
	{
		$this->db->select('ea.company_name, ea.domain_url, concat(eu.first_name, " ", eu.last_name) as user_name', false);
		$this->db->from('entity_accounts as ea');
		$this->db->join('entity_users as eu', 'ea.entity_id = eu.entity_id');
		$this->db->where('eu.email',trim($to_email));
        $query = $this->db->get();

        $entity = $query->result_array();

        if (!$entity) {
            $this->error = lang('INVALID_EMAIL');
            return false;
        }

		$this->load->library('email');
		$msg = '';
		foreach ($entity as $row) {
			$urls = explode(',',$row['domain_url']);
			if (count($urls)>1){
				foreach($urls as $url) {
					$msg .= $url." (".$row['company_name'].") "."\n\n";
				}		
			} else {
				$msg .= $row['domain_url']." (".$row['company_name'].") "."\n\n";
			}
		}

		$msg = sprintf(lang("UPGRADE_YOUR_VERSION_MSG"),$entity[0]['user_name'],$msg,$to_email);

		$email_config = array(
		    'mailtype'    => 'html',
		    'charset'    => 'utf-8'
		);

		$from['name']	= $this->config->item('domain_name');
		$from['email']	= $this->config->item('smtp_user');

		$this->email->initialize($email_config);	
        $email = $this->email;
        $email->clear();
        $email->to($to_email);
        $email->from($from['email'],$from['name']);
        $email->subject(lang("ENTITY_DOMAIN_URL_REQUEST"));
        $email->message(nl2br($msg));

        if (!$email->send())
		{
			$this->error = $email->print_debugger();
			return false;
		}

        return true;
	}
}
